"""
文件读写相关工具
"""
from .base import BaseTool


class FileRead(BaseTool):
    name = "FileRead"
    description = "读取文件内容"
    arguments = {"path": "文件路径"}
    def execute(self, path):
        with open(path, "r", encoding="utf-8") as f:
            return f.read()

class FileWrite(BaseTool):
    name = "FileWrite"
    description = "写入或覆盖文件内容（高风险，需确认）"
    arguments = {"path": "文件路径", "content": "写入内容"}
    def execute(self, path, content):
        with open(path, "w", encoding="utf-8") as f:
            f.write(content)
        return f"写入完成: {path}"

class FileEdit(BaseTool):
    name = "FileEdit"
    description = "编辑文件内容（如按行替换，需确认）"
    arguments = {"path": "文件路径", "instructions": "编辑说明"}
    def execute(self, path, instructions):
        # TODO: 实现按指令编辑文件
        return "暂未实现"



"""
目录导航工具
"""
import os

from .base import BaseTool


class Ls(BaseTool):
    name = "Ls"
    description = "列出目录内容"
    arguments = {"path": "目录路径"}
    def execute(self, path="."):
        try:
            return os.listdir(path)
        except Exception as e:
            return f"列出目录失败: {e}"



"""安全框架模块

提供安全管理器和相关组件，用于管理操作权限和安全策略。
"""

from .security import SecurityManager, SecurityLevel, ToolRiskLevel, confirm_action

__all__ = [
    'SecurityManager',
    'SecurityLevel',
    'ToolRiskLevel',
    'confirm_action',
]


"""
工具基类/接口
"""
from typing import Dict, Any

class BaseTool:
    name: str
    description: str
    arguments: Dict[str, Any]

    def execute(self, **kwargs):
        """执行工具的核心逻辑
        
        Args:
            **kwargs: 工具执行所需的参数
            
        Returns:
            Any: 工具执行结果
        """
        raise NotImplementedError()



"""
git 相关工具集合
"""
import subprocess
from .base import BaseTool
from typing import Optional

class GitStatus(BaseTool):
    name = "GitStatus"
    description = "显示当前工作区的 git 状态"
    arguments = {}
    def execute(self):
        try:
            result = subprocess.run(["git", "status"], capture_output=True, text=True, timeout=20, check=False)
            return result.stdout or result.stderr
        except Exception as e:
            return f"执行出错: {e}"

class GitLog(BaseTool):
    name = "GitLog"
    description = "显示 git 提交日志"
    arguments = {"max_count": {"type": "int", "required": False, "desc": "显示的最大提交数"}}
    def execute(self, max_count: Optional[int] = None):
        try:
            cmd = ["git", "log"]
            if max_count is not None:
                if not isinstance(max_count, int):
                    return "参数 max_count 必须为整数"
                cmd += ["-n", str(max_count)]
            result = subprocess.run(cmd, capture_output=True, text=True, timeout=20, check=False)
            return result.stdout or result.stderr
        except Exception as e:
            return f"执行出错: {e}"

class GitDiff(BaseTool):
    name = "GitDiff"
    description = "显示当前工作区的 git diff"
    arguments = {"path": {"type": "str", "required": False, "desc": "指定 diff 路径"}}
    def execute(self, path: Optional[str] = None):
        try:
            cmd = ["git", "diff"]
            if path:
                if not isinstance(path, str):
                    return "参数 path 必须为字符串"
                cmd.append(path)
            result = subprocess.run(cmd, capture_output=True, text=True, timeout=20, check=False)
            return result.stdout or result.stderr
        except Exception as e:
            return f"执行出错: {e}"

class GitBranch(BaseTool):
    name = "GitBranch"
    description = "显示或切换 git 分支"
    arguments = {"branch": {"type": "str", "required": False, "desc": "切换到的分支名"}}
    def execute(self, branch: Optional[str] = None):
        try:
            if branch:
                if not isinstance(branch, str):
                    return "参数 branch 必须为字符串"
                cmd = ["git", "checkout", branch]
            else:
                cmd = ["git", "branch"]
            result = subprocess.run(cmd, capture_output=True, text=True, timeout=20, check=False)
            return result.stdout or result.stderr
        except Exception as e:
            return f"执行出错: {e}"

class GitPull(BaseTool):
    name = "GitPull"
    description = "执行 git pull 拉取远程更新"
    arguments = {
        "remote": {"type": "str", "required": False, "desc": "远程仓库名"},
        "branch": {"type": "str", "required": False, "desc": "分支名"}
    }
    def execute(self, remote: Optional[str] = None, branch: Optional[str] = None):
        try:
            cmd = ["git", "pull"]
            if remote:
                if not isinstance(remote, str):
                    return "参数 remote 必须为字符串"
                cmd.append(remote)
            if branch:
                if not isinstance(branch, str):
                    return "参数 branch 必须为字符串"
                cmd.append(branch)
            result = subprocess.run(cmd, capture_output=True, text=True, timeout=30, check=False)
            return result.stdout or result.stderr
        except Exception as e:
            return f"执行出错: {e}"

class GitPush(BaseTool):
    name = "GitPush"
    description = "执行 git push 推送本地提交到远程"
    arguments = {
        "remote": {"type": "str", "required": False, "desc": "远程仓库名"},
        "branch": {"type": "str", "required": False, "desc": "分支名"}
    }
    def execute(self, remote: Optional[str] = None, branch: Optional[str] = None):
        try:
            cmd = ["git", "push"]
            if remote:
                if not isinstance(remote, str):
                    return "参数 remote 必须为字符串"
                cmd.append(remote)
            if branch:
                if not isinstance(branch, str):
                    return "参数 branch 必须为字符串"
                cmd.append(branch)
            result = subprocess.run(cmd, capture_output=True, text=True, timeout=30, check=False)
            return result.stdout or result.stderr
        except Exception as e:
            return f"执行出错: {e}"

class GitAdd(BaseTool):
    name = "GitAdd"
    description = "添加文件到暂存区"
    arguments = {
        "paths": {"type": "list[str]", "required": True, "desc": "要添加的文件路径列表，至少一个"}
    }
    def execute(self, paths):
        if not paths or not isinstance(paths, list) or not all(isinstance(p, str) for p in paths):
            return "参数 paths 必须为字符串列表，且不能为空"
        try:
            cmd = ["git", "add"] + paths
            result = subprocess.run(cmd, capture_output=True, text=True, timeout=20, check=False)
            return result.stdout or result.stderr or "添加成功"
        except Exception as e:
            return f"执行出错: {e}"

class GitCommit(BaseTool):
    name = "GitCommit"
    description = "提交暂存区到仓库"
    arguments = {
        "message": {"type": "str", "required": True, "desc": "提交信息"},
        "all": {"type": "bool", "required": False, "desc": "是否 git commit -a"}
    }
    def execute(self, message, all: Optional[bool] = False):
        if not message or not isinstance(message, str):
            return "参数 message 必须为字符串，且不能为空"
        try:
            cmd = ["git", "commit", "-m", message]
            if all:
                cmd.insert(2, "-a")
            result = subprocess.run(cmd, capture_output=True, text=True, timeout=20, check=False)
            return result.stdout or result.stderr or "提交成功"
        except Exception as e:
            return f"执行出错: {e}"

class GitTag(BaseTool):
    name = "GitTag"
    description = "创建或列出 git 标签"
    arguments = {
        "name": {"type": "str", "required": False, "desc": "标签名，为空则列出所有标签"},
        "message": {"type": "str", "required": False, "desc": "标签说明，创建标签时可选"}
    }
    def execute(self, name: Optional[str] = None, message: Optional[str] = None):
        try:
            if name:
                if not isinstance(name, str):
                    return "参数 name 必须为字符串"
                cmd = ["git", "tag", name]
                if message:
                    if not isinstance(message, str):
                        return "参数 message 必须为字符串"
                    cmd += ["-m", message]
            else:
                cmd = ["git", "tag"]
            result = subprocess.run(cmd, capture_output=True, text=True, timeout=20, check=False)
            return result.stdout or result.stderr
        except Exception as e:
            return f"执行出错: {e}"

class GitRebase(BaseTool):
    name = "GitRebase"
    description = "变基操作（rebase）"
    arguments = {
        "branch": {"type": "str", "required": True, "desc": "要 rebase 的目标分支名"}
    }
    def execute(self, branch):
        if not branch or not isinstance(branch, str):
            return "参数 branch 必须为字符串，且不能为空"
        try:
            cmd = ["git", "rebase", branch]
            result = subprocess.run(cmd, capture_output=True, text=True, timeout=30, check=False)
            return result.stdout or result.stderr
        except Exception as e:
            return f"执行出错: {e}"



"""安全层 - 集中处理危险操作的用户确认逻辑

该模块提供安全确认机制，用于在执行高风险操作前获取用户确认，
并根据配置的安全级别决定是否需要确认或完全禁用某些操作。
"""

import os
from enum import Enum, auto
from pathlib import Path
from typing import Any, Dict, List, Optional

try:
    from rich.console import Console
    from rich.prompt import Confirm
    RICH_AVAILABLE = True
except ImportError:
    RICH_AVAILABLE = False


class SecurityLevel(Enum):
    """安全级别枚举
    
    - STRICT: 最严格模式，所有高风险操作都需要确认，某些操作可能被禁用
    - NORMAL: 标准模式，高风险操作需要确认
    - RELAXED: 宽松模式，只有极高风险操作需要确认
    - BYPASS: 跳过所有确认（危险！仅用于自动化脚本）
    """
    STRICT = auto()
    NORMAL = auto()
    RELAXED = auto()
    BYPASS = auto()


class ToolRiskLevel(Enum):
    """工具风险级别枚举
    
    - NONE: 无风险，如只读操作
    - LOW: 低风险，如创建新文件
    - MEDIUM: 中等风险，如修改文件
    - HIGH: 高风险，如删除文件
    - CRITICAL: 极高风险，如执行Shell命令
    """
    NONE = auto()
    LOW = auto()
    MEDIUM = auto()
    HIGH = auto()
    CRITICAL = auto()


class SecurityManager:
    """安全管理器，处理所有安全相关的逻辑"""
    
    def __init__(
        self,
        security_level: SecurityLevel = SecurityLevel.NORMAL,
        allowed_paths: Optional[List[str]] = None,
        disabled_tools: Optional[List[str]] = None,
        yes_to_all: bool = False
    ):
        """初始化安全管理器
        
        Args:
            security_level: 安全级别
            allowed_paths: 允许操作的路径列表，None表示不限制
            disabled_tools: 禁用的工具列表
            yes_to_all: 是否对所有确认自动回答是（危险！）
        """
        self.security_level = security_level
        self.allowed_paths = [Path(p).expanduser().resolve() for p in (allowed_paths or [])]
        self.disabled_tools = disabled_tools or []
        self.yes_to_all = yes_to_all
        self.console = Console() if RICH_AVAILABLE else None
        
        # 各安全级别下需要确认的工具风险等级
        self._confirm_thresholds = {
            SecurityLevel.STRICT: ToolRiskLevel.LOW,      # 严格模式：低风险及以上需确认
            SecurityLevel.NORMAL: ToolRiskLevel.MEDIUM,  # 标准模式：中风险及以上需确认
            SecurityLevel.RELAXED: ToolRiskLevel.HIGH,   # 宽松模式：高风险及以上需确认
            SecurityLevel.BYPASS: ToolRiskLevel.CRITICAL # 跳过模式：仅极高风险需确认
        }
        
        # 各安全级别下禁用的工具风险等级
        self._disable_thresholds = {
            SecurityLevel.STRICT: ToolRiskLevel.CRITICAL,  # 严格模式：禁用极高风险
            SecurityLevel.NORMAL: None,                  # 标准模式：不自动禁用
            SecurityLevel.RELAXED: None,                 # 宽松模式：不自动禁用
            SecurityLevel.BYPASS: None                   # 跳过模式：不自动禁用
        }
    
    def is_tool_allowed(self, tool_name: str, risk_level: ToolRiskLevel) -> bool:
        """检查工具是否被允许使用
        
        Args:
            tool_name: 工具名称
            risk_level: 工具风险级别
            
        Returns:
            bool: 是否允许使用该工具
        """
        # 检查工具是否在禁用列表中
        if tool_name in self.disabled_tools:
            return False
            
        # 检查工具风险级别是否超过当前安全级别的禁用阈值
        disable_threshold = self._disable_thresholds[self.security_level]
        if disable_threshold and risk_level.value >= disable_threshold.value:
            return False
            
        return True
    
    def is_path_allowed(self, path: str) -> bool:
        """检查路径是否在允许的范围内
        
        Args:
            path: 要检查的路径
            
        Returns:
            bool: 如果路径在允许范围内返回True，否则返回False
        """
        # 如果没有设置允许路径，则所有路径都允许
        if not self.allowed_paths:
            return True
            
        # 将路径转换为绝对路径
        abs_path = Path(path).expanduser().resolve()
        
        # 检查路径是否在允许的路径列表中
        for allowed_path in self.allowed_paths:
            if abs_path == allowed_path or allowed_path in abs_path.parents:
                return True
                
        return False
    
    def confirm_action(
        self,
        message: str,
        risk_level: ToolRiskLevel,
        details: Optional[Dict[str, Any]] = None
    ) -> bool:
        """请求用户确认操作
        
        Args:
            message: 确认消息
            risk_level: 操作风险级别
            details: 操作详情，用于展示给用户
            
        Returns:
            bool: 用户是否确认执行操作
        """
        # 如果设置了yes_to_all，直接返回True
        if self.yes_to_all:
            return True
            
        # 检查当前安全级别下是否需要确认该风险级别的操作
        confirm_threshold = self._confirm_thresholds[self.security_level]
        if risk_level.value < confirm_threshold.value:
            return True  # 风险级别低于阈值，无需确认
        
        # 格式化详情信息
        detail_str = ""
        if details:
            detail_str = "\n详情:\n" + "\n".join(f"  {k}: {v}" for k, v in details.items())
        
        # 使用rich库进行美化确认（如果可用）
        if RICH_AVAILABLE:
            self.console.print(f"[bold yellow]安全确认[/bold yellow]: {message}")
            if detail_str:
                self.console.print(detail_str)
            return Confirm.ask("是否继续?")
        else:
            # 回退到基本确认方式
            print(f"安全确认: {message}{detail_str}")
            resp = input("是否继续? [y/N]: ").strip().lower()
            return resp == "y"


# 默认安全管理器实例
security_manager = SecurityManager()


# 向后兼容的函数
def confirm_action(message: str) -> bool:
    """请求用户确认操作（向后兼容）
    
    Args:
        message: 确认消息
        
    Returns:
        bool: 用户是否确认执行操作
    """
    return security_manager.confirm_action(
        message=message,
        risk_level=ToolRiskLevel.MEDIUM
    )


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
简单测试重构后的 LLM 接口核心功能
"""

import os
import sys
from typing import Dict, Any, Optional, List

# 创建一个简单的模拟类来测试核心功能
class MockOpenAIBasedLLM:
    """模拟 OpenAIBasedLLM 类的核心功能"""
    
    def __init__(self, config: Optional[Dict[str, Any]] = None):
        self.config = config or {}
        self.api_key = self.config.get("api_key")
        self.model = self.config.get("model")
        self.api_base = self.config.get("endpoint")
        self.extra_params = self.config.get("extra_params", {})
        self.verify_ssl = self._parse_verify_ssl_config()
        
    def _parse_verify_ssl_config(self) -> bool:
        """从配置中解析 SSL 验证设置"""
        if "verify_ssl" in self.config:
            _verify_ssl_raw = self.config["verify_ssl"]
        else:
            return True
            
        if isinstance(_verify_ssl_raw, bool):
            return _verify_ssl_raw
        elif isinstance(_verify_ssl_raw, str):
            return _verify_ssl_raw.strip().lower() in ("true", "1", "yes")
        else:
            return bool(_verify_ssl_raw)
    
    def _handle_api_error(self, error: Exception, provider_name: str = "API") -> str:
        """通用 API 错误处理逻辑"""
        error_msg = str(error)
        return f"Error: {error_msg}"
    
    def _process_completion_response(self, completion, stream=False):
        """处理 API 返回的 completion 对象"""
        # 简单模拟返回内容
        return "这是一个模拟的 API 响应。"
    
    def generate(self, prompt: str, **kwargs) -> str:
        """使用聊天接口模拟 generate 功能"""
        messages = [{"role": "user", "content": prompt}]
        return self.chat(messages, **kwargs)
    
    def chat(self, messages: List[Dict[str, str]], **kwargs) -> str:
        """实现通用的 chat 接口"""
        try:
            # 获取参数
            max_tokens = kwargs.get("max_tokens", self.config.get("max_tokens", 2048))
            temperature = kwargs.get("temperature", self.config.get("temperature", 0.7))
            
            # 模拟 API 调用
            print(f"模拟 API 调用:")
            print(f"- 模型: {self.model}")
            print(f"- 消息: {messages}")
            print(f"- 最大 token 数: {max_tokens}")
            print(f"- 温度: {temperature}")
            print(f"- SSL 验证: {self.verify_ssl}")
            
            # 模拟响应
            return f"这是对 '{messages[-1]['content']}' 的模拟响应。"
            
        except Exception as error:
            # 使用通用错误处理
            return self._handle_api_error(error, "模拟 API")


class MockOpenAILLM(MockOpenAIBasedLLM):
    """模拟 OpenAI 接口"""
    def __init__(self, config: Optional[Dict[str, Any]] = None):
        super().__init__(config)


class MockVolcEngineLLM(MockOpenAIBasedLLM):
    """模拟火山引擎接口"""
    DEFAULT_BASE_URL = "https://ark.cn-beijing.volces.com/api/v3"
    
    def __init__(self, config: Optional[Dict[str, Any]] = None):
        # 从环境变量获取 API Key
        config = config or {}
        if not config.get("api_key"):
            api_key = os.environ.get("ARK_API_KEY")
            if api_key:
                config["api_key"] = api_key
        
        super().__init__(config)
        
        # 模型 ID 是必须的
        if not self.model:
            raise ValueError("未在配置中指定火山引擎模型 (model)。")
    
    def chat(self, messages: List[Dict[str, str]], **kwargs) -> str:
        # 火山引擎的默认 max_tokens 与 OpenAI 不同
        if "max_tokens" not in kwargs and "max_tokens" not in self.config:
            kwargs["max_tokens"] = 500  # 火山引擎默认值
            
        # 使用父类的 chat 方法
        return super().chat(messages, **kwargs)


def test_openai_mock():
    """测试 OpenAI 模拟接口"""
    print("=== 测试 OpenAI 模拟接口 ===")
    
    config = {
        "api_key": "sk-example",
        "model": "gpt-3.5-turbo",
        "max_tokens": 100
    }
    
    llm = MockOpenAILLM(config)
    messages = [{"role": "user", "content": "你好，请简单介绍一下自己。"}]
    response = llm.chat(messages)
    print(f"响应: {response}")
    print("测试成功!")


def test_volcengine_mock():
    """测试火山引擎模拟接口"""
    print("\n=== 测试火山引擎模拟接口 ===")
    
    config = {
        "api_key": "ark-example",
        "model": "eb-4",
        "verify_ssl": False,
        "max_tokens": 100
    }
    
    llm = MockVolcEngineLLM(config)
    messages = [{"role": "user", "content": "你好，请简单介绍一下自己。"}]
    response = llm.chat(messages)
    print(f"响应: {response}")
    print("测试成功!")


if __name__ == "__main__":
    test_openai_mock()
    test_volcengine_mock()



"""功能层 - 提供核心功能实现

包含代码编辑、命令处理、Git集成、Shell集成和代理循环架构。
"""

from .agent.agent import NezhaAgent
from .commands.chat_command import ChatCommand
from .commands.plan_command import PlanCommand

__all__ = [
    'NezhaAgent',
    'ChatCommand',
    'PlanCommand',
]